#-- 
#  Copyright (C) 2008 Red Hat Inc.
#  
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#  
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#  
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# Author: Bryan Kearney <bkearney@redhat.com>
#--

require "yaml"
require "net/http"
require "puppet"

module Ace
	module Util
	
		# Sends an HTTP_POST and follows basic redirection
		def post_url(uri, parameters) 
			url = URI.parse(uri)
			info "Using URL #{url} with parameters #{parameters.inspect}"
			if (url.query.nil?)
				req = Net::HTTP::Post.new(url.path)
			else
				req = Net::HTTP::Post.new(url.path+"?"+url.query)
			end
			req.set_form_data(parameters)
			res = Net::HTTP.new(url.host).start {|http| http.request(req) }	
			while res.is_a?(Net::HTTPRedirection)
				info "Redirecting to {res['location']}"
				res = get_url(res['location']) 
			end
			return [res.is_a?(Net::HTTPSuccess), res.code]
		end
		module_function :post_url
		
		# Sends an http get
		def get_url(uri)
			url = URI.parse(uri)
			req = Net::HTTP::Get.new(url.path+"?"+url.query)
			res = Net::HTTP.new(url.host).start {|http| http.request(req) }	
		end
		module_function :get_url		
	
	
		# Use YAML to serialize and hash into and out of a string.
		def serialize_hash(hash)
			return hash.to_yaml
		end
		module_function :serialize_hash		
		
		# Takes in YAML or an array of "KEY=>VALUE" strings
		def deserialize_hash(data)
			return_value = Hash.new()
			if data.is_a?(String)
				return_value = YAML.load(data)
			elsif data.is_a?(Array)
				data.each() do |datum|
					data_items = datum.split("=>")
					return_value[data_items[0]]=data_items[1]
				end
			end
			return return_value
		end
		module_function :deserialize_hash				
		
        def clean_exec(cmd, user=nil, group=nil, quiet=true)
            ouput_file_name="ace-#{Kernel.rand(10000)}.tmp"
            output_file = Tempfile.new(ouput_file_name)		
            output = nil			
            pid = Kernel.fork
            uid = Process.uid
            gid = Process.gid		
            if pid
                # parent
                status = Process.wait
            else
                # child
                if user
                    uid = user
                    gid = group
                    if (user.class == String)
                        uid = User::get_uid(user)
                        gid = User::get_gid(user) if ! gid
                    end				
                end
                if quiet
                    STDOUT.reopen(output_file.path)
                    STDERR.reopen(output_file.path)
                end
                Process.setsid	
                Process.uid = uid if uid		
                Process.gid = gid if gid
                Process.euid = uid if uid				
                Kernel.exec(cmd)
            end
            status = $?.exitstatus
            output = ""
            if quiet
                output = File.read(output_file.path)
            end
            return output, status
        end
        module_function :clean_exec		
	end
end
