#-- 
#  Copyright (C) 2008 Red Hat Inc.
#  
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#  
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#  
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# Author: Bryan Kearney <bkearney@redhat.com>
#--

Puppet::Type.newtype(:file_append) do
	@doc = "Append a line in a file if it is not present"
	
	newparam (:name) do
		desc "My Name"
	end
	
	newparam(:file) do
		desc "The file to replace"
	end

	newparam(:line) do
		desc "The line to append"
	end

	newproperty(:ensure) do
		desc "Whether the resource is in sync or not."

		defaultto :insync
		
		# Make output a bit prettier
		def change_to_s(currentvalue, newvalue)
			return "executed successfully"
		end		

		def retrieve
			if !(File.exist?(resource[:file]))
				:outofsync
			else      		  
		  		data = File.read(resource[:file])
		  		return (data.include?(resource[:line])) ? :insync : :outofsync
		  	end
		end

      newvalue :outofsync
      newvalue :insync do
			if !(File.exist?(resource[:file]))
				fail("File #{resource[:file]} does not exist")
			else      
				file = File.open(resource[:file], "a")
				file.puts(resource[:line])
				notice("Appended #{resource[:line]} into #{resource[:file]}")				
			end 
      end
    end
	
 	autorequire :file do
		[self[:file]]
	end	
end
